package at.ac.tuwien.sepm.assignment.individual.persistence.impl;

import at.ac.tuwien.sepm.assignment.individual.persistence.FileDao;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Repository;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.nio.file.FileSystemException;
import java.nio.file.Files;

@Repository
public class HorseFileDao implements FileDao {
    @Value("${spring.upload.path}")
    private String FILE_BASE_PATH;
    private static final Logger LOGGER = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    public HorseFileDao() {};

    @Override
    public void save(MultipartFile file) throws IOException {
        if(file == null || file.getOriginalFilename() == null || file.isEmpty())
            throw new IOException("Cannot save an empty file");

        if(!file.getContentType().equals("image/jpeg") && !file.getContentType().equals("image/png"))
            throw new IOException("Unsupported file type");

        String fileName = file.getOriginalFilename();
        LOGGER.trace("Writing file to " + FILE_BASE_PATH + fileName);

        try {
            // Create directory if it doesn't exist
            File uploadDir = new File(FILE_BASE_PATH);
            if(!uploadDir.exists())
                uploadDir.mkdir();

            // Save the file
            if(uploadDir.exists()) {
                File target = new File(FILE_BASE_PATH + fileName);
                Files.write(target.toPath(), file.getBytes());
            } else {
                throw new FileSystemException("Base upload directory " + FILE_BASE_PATH + " does not exist");
            }
        } catch (Exception e) {
            LOGGER.error(e.getMessage());
            throw new IOException("Saving the file failed");
        }
    }

    @Override
    public void delete(String fileName) throws IOException {
        if(fileName == null || fileName.equals(""))
            throw new IOException("Cannot delete an unexisting file");

        LOGGER.trace("Deleting file from " + FILE_BASE_PATH + fileName);
        try {
            new File(FILE_BASE_PATH + fileName).delete();
        } catch(Exception e) {
            LOGGER.error(e.getMessage());
            throw new IOException("Deleting the file specified failed");
        }
    }

    @Override
    public byte[] get(String fileName) throws IOException {
        if(fileName == null || fileName.equals(""))
            throw new IOException("Cannot open an unexisting file");

        LOGGER.trace("Getting file from " + FILE_BASE_PATH + fileName);
        try {
            // Read the file and return a byte array
            // https://stackoverflow.com/a/5083666
            File imageFile = new File(FILE_BASE_PATH + fileName);
            return Files.readAllBytes(imageFile.toPath());
        } catch(Exception e) {
            LOGGER.error(e.getMessage());
            throw new IOException("Opening the file specified failed");
        }
    }
}
