package at.ac.tuwien.sepm.assignment.individual.service;

import at.ac.tuwien.sepm.assignment.individual.entity.Horse;
import at.ac.tuwien.sepm.assignment.individual.entity.Owner;
import at.ac.tuwien.sepm.assignment.individual.exception.NotFoundException;
import at.ac.tuwien.sepm.assignment.individual.util.ValidationException;
import org.springframework.dao.DataAccessException;

import java.util.List;
import java.util.Map;

public interface OwnerService {


    /**
     * @param id of the owner to find.
     * @return the owner with the specified id.
     * @throws RuntimeException  will be thrown if something goes wrong during data processing.
     * @throws NotFoundException will be thrown if the owner could not be found in the system.
     */
    Owner findOneById(Long id);

    /**
     * @return a list of all owners in the database
     * @throws NotFoundException will be thrown if there are no owners in the database
     */
    List<Owner> getAll() throws NotFoundException;

    /**
     * @param filters to use for filtering the owners
     * @return a list of all owners that fill the criteria
     * @throws NotFoundException   will be thrown if no owners fill the criteria
     * @throws ValidationException will be thrown if the filter contains bad values
     */
    List<Owner> getFiltered(Map<String, String> filters) throws NotFoundException, ValidationException;

    /**
     * @param id of the owner whose horses the database should obtain
     * @return a list of all horses that the specified owner owns
     * @throws NotFoundException will be thrown if the owner owns no horses or if the owner could not be found
     * @throws DataAccessException will be thrown if something goes wrong during the database access
     */
    List<Horse> getOwnedHorses(Long id) throws NotFoundException;

    /**
     * @param owner to create
     * @return the new owner
     * @throws ValidationException will be thrown if something goes wrong during verification.
     * @throws DataAccessException will be thrown if the owner could not be saved in the database.
     */
    Owner addOwner(Owner owner) throws ValidationException, DataAccessException;

    /**
     * @param owner that specifies the new owner values alongside with the id of the owner to update
     * @return the updated owner
     * @throws ValidationException will be thrown if something goes wrong during verification.
     * @throws DataAccessException will be thrown if the owner could not be saved in the database.
     */
    Owner updateOwner(Owner owner) throws ValidationException, DataAccessException;

    /**
     * @param id of the owner to delete
     * @throws NotFoundException   will be thrown if the owner could not be found in the system
     * @throws DataAccessException will be thrown if the owner could not be deleted from the database
     */
    void deleteOwner(Long id) throws NotFoundException, DataAccessException;
}
