package at.ac.tuwien.sepm.assignment.individual.service.impl;

import at.ac.tuwien.sepm.assignment.individual.entity.Horse;
import at.ac.tuwien.sepm.assignment.individual.exception.NotFoundException;
import at.ac.tuwien.sepm.assignment.individual.persistence.FileDao;
import at.ac.tuwien.sepm.assignment.individual.persistence.HorseDao;
import at.ac.tuwien.sepm.assignment.individual.service.HorseService;
import at.ac.tuwien.sepm.assignment.individual.util.ValidationException;
import at.ac.tuwien.sepm.assignment.individual.util.Validator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.lang.invoke.MethodHandles;
import java.util.List;
import java.util.Map;

@Service
public class SimpleHorseService implements HorseService {
    private static final Logger LOGGER = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    private final HorseDao horseJdbcDao;
    private final FileDao horseFileDao;
    private final Validator validator;

    @Autowired
    public SimpleHorseService(HorseDao horseJdbcDao, FileDao horseFileDao, Validator validator) {
        this.horseJdbcDao = horseJdbcDao;
        this.horseFileDao = horseFileDao;
        this.validator = validator;
    }

    @Override
    public Horse findOneById(Long id) {
        LOGGER.trace("findOneById({})", id);
        return horseJdbcDao.findOneById(id);
    }

    @Override
    public List<Horse> getAll() throws NotFoundException {
        LOGGER.trace("getAll()");
        return horseJdbcDao.getAll();
    }

    @Override
    public List<Horse> getFiltered(Map<String, String> filters) throws NotFoundException, ValidationException {
        LOGGER.trace("getFiltered({})", filters.entrySet());
        this.validator.validateHorseFilter(filters);
        return horseJdbcDao.getFiltered(filters);
    }

    @Override
    public Horse addHorse(Horse horse) throws ValidationException, DataAccessException {
        this.validator.validateNewHorse(horse);
        LOGGER.trace("addHorse({})", horse.toString());
        return horseJdbcDao.addHorse(horse);
    }

    @Override
    public Horse updateHorse(Horse horse) throws ValidationException, DataAccessException, IOException {
        this.validator.validateUpdateHorse(horse);
        LOGGER.trace("updateHorse({})", horse.toString());
        return horseJdbcDao.updateHorse(horse);
    }

    @Override
    public void deleteHorse(Long id) throws NotFoundException, DataAccessException, IOException {
        LOGGER.trace("deleteHorse({})", id);
        horseJdbcDao.deleteHorse(id);
    }

    @Override
    public void saveImage(MultipartFile img) throws ValidationException, IOException {
        this.validator.validateHorseImage(img);
        LOGGER.trace("saveImage({})", img.getName());
        horseFileDao.save(img);
    }

    @Override
    public byte[] getImage(String path) throws IOException, ValidationException {
        this.validator.validateImageRequest(path);
        LOGGER.trace("getImage({})", path);
        return horseFileDao.get(path);
    }
}
