package at.ac.tuwien.sepm.assignment.individual.service.impl;

import at.ac.tuwien.sepm.assignment.individual.entity.Horse;
import at.ac.tuwien.sepm.assignment.individual.entity.Owner;
import at.ac.tuwien.sepm.assignment.individual.exception.NotFoundException;
import at.ac.tuwien.sepm.assignment.individual.persistence.OwnerDao;
import at.ac.tuwien.sepm.assignment.individual.service.OwnerService;
import at.ac.tuwien.sepm.assignment.individual.util.ValidationException;
import at.ac.tuwien.sepm.assignment.individual.util.Validator;

import java.lang.invoke.MethodHandles;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.stereotype.Service;

@Service
public class SimpleOwnerService implements OwnerService {

    private static final Logger LOGGER = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    private final OwnerDao ownerDao;
    private final Validator validator;

    @Autowired
    public SimpleOwnerService(OwnerDao ownerDao, Validator validator) {
        this.ownerDao = ownerDao;
        this.validator = validator;
    }

    @Override
    public Owner findOneById(Long id) {
        LOGGER.trace("findOneById({})", id);
        return ownerDao.findOneById(id);
    }

    @Override
    public List<Owner> getAll() throws NotFoundException {
        LOGGER.trace("getAll()");
        return ownerDao.getAll();
    }

    @Override
    public List<Owner> getFiltered(Map<String, String> filters) throws NotFoundException, ValidationException {
        LOGGER.trace("getFiltered({})", filters.entrySet());
        this.validator.validateOwnerFilter(filters);
        return ownerDao.getFiltered(filters);
    }

    @Override
    public List<Horse> getOwnedHorses(Long id) throws NotFoundException {
        LOGGER.trace("getOwnedHorses({})", id);
        return ownerDao.getOwnedHorses(id);
    }

    @Override
    public Owner addOwner(Owner owner) throws ValidationException, DataAccessException {
        LOGGER.trace("addOwner({})", owner);
        this.validator.validateNewOwner(owner);
        return ownerDao.addOwner(owner);
    }

    @Override
    public Owner updateOwner(Owner owner) throws ValidationException, DataAccessException {
        LOGGER.trace("updateOwner({})", owner);
        this.validator.validateUpdateOwner(owner);
        return ownerDao.updateOwner(owner);
    }

    @Override
    public void deleteOwner(Long id) throws NotFoundException, DataAccessException {
        LOGGER.trace("deleteOwner({})", id);
        ownerDao.deleteOwner(id);
    }
}
