import { Component, OnInit } from '@angular/core';
import { Owner } from '../../dto/owner';
import { OwnerService } from '../../service/owner.service';
import { HttpParams } from '@angular/common/http';

@Component({
  selector: 'app-list-owners',
  templateUrl: './list-owners.component.html',
  styleUrls: ['./list-owners.component.scss']
})
export class ListOwnersComponent implements OnInit {
  error = false;
  errorMessage = '';
  owners: Array<Owner>;
  filters: {
    name: string
  };

  constructor(private ownerService: OwnerService) { }

  ngOnInit(): void {
    // Nullify queries
    this.filters = {
      name: ""
    };
    this.loadAllOwners();
  }

  /**
   * Error flag will be deactivated, which clears the error message
   */
  vanishError() {
    this.error = false;
  }

  /**
   * Loads all owners from the database
   */
  private loadAllOwners() {
    console.log("GET all owners from API");
    this.ownerService.getAllOwners().subscribe(
      (owners: Array<Owner>) => {
        this.owners = owners;
      },
      error => {
        this.defaultServiceErrorHandling(error);
      }
    );
  }

  /**
   * Load filtered owners from the database
   */
  public loadFilteredOwners() {
    console.log("GET filtered owners from API");

    console.log(this.filters);
    // Create HttpParams for the sorting
    let params: HttpParams = new HttpParams();
    if(this.filters.name != null && this.filters.name.length !== 0)
      params = params.set('name', this.filters.name);

    this.ownerService.getFilteredOwners(params).subscribe(
      (owners: Array<Owner>) => {
        this.owners = owners;
      },
      error => {
        this.owners = null;
        this.defaultServiceErrorHandling(error);
      }
    );
  }

  public deleteOwner(id: number, name: string) {
    if(confirm("Are you sure you want to delete " + name + ". This action is irreversible.")) {
      this.ownerService.deleteOwner(id).subscribe(
        (res) => {
          console.log('Successfully deleted owner');
          this.loadAllOwners();
        }, (error) => {
          this.defaultServiceErrorHandling(error);
        }
      );
    }
  }

  private defaultServiceErrorHandling(error: any) {
    console.log(error);
    this.error = true;
    if (error.status === 0) {
      // If status is 0, the backend is probably down
      this.errorMessage = 'The backend seems not to be reachable';
    } else if (error.error.message === 'No message available') {
      // If no detailed error message is provided, fall back to the simple error name
      this.errorMessage = error.error.error;
    } else {
      this.errorMessage = error.error.message;
    }
  }

}
