import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Params} from '@angular/router';
import { Owner } from '../../dto/owner';
import { OwnerService } from '../../service/owner.service';

@Component({
  selector: 'app-update-owner',
  templateUrl: './update-owner.component.html',
  styleUrls: ['./update-owner.component.scss']
})
export class UpdateOwnerComponent implements OnInit {
  error = false;
  success: string;
  errorMessage = '';
  owner: Owner = new Owner(null, null);

  constructor(private ownerService: OwnerService, private route: ActivatedRoute) { }

  ngOnInit(): void {
    // Extract id from url
    const ownerId: string = this.route.snapshot.paramMap.get('id');

    // Get current value of the owner and save it
    this.ownerService.getOwnerById(parseInt(ownerId, 10)).subscribe(
      (owner: Owner) => {
        this.owner = owner;
      },
      error => {
        this.defaultServiceErrorHandling(error);
      }
    );
  }

  /**
   * Will be called on a click on the success alert close button
   */
  public vanishAlert() {
    this.success = null;
  }

  /**
   * Will be called on a click on the error alert close button
   */
  public vanishError() {
    this.errorMessage = null;
  }

  /**
   * Updates a owner and loads it
   * @param owner
   */
  public updateOwner() {
    console.log("PUT owner to API")
    console.log(this.owner);
    this.ownerService.updateOwner(this.owner).subscribe(
      (result: Owner) => {
        this.success = result.name;
      },
      error => {
        this.defaultServiceErrorHandling(error);
      }
    );
  }

  private defaultServiceErrorHandling(error: any) {
    console.log(error);
    this.error = true;
    if (error.status === 0) {
      // If status is 0, the backend is probably down
      this.errorMessage = 'The backend seems not to be reachable';
    } else if (error.error.message === 'No message available') {
      // If no detailed error message is provided, fall back to the simple error name
      this.errorMessage = error.error.error;
    } else {
      this.errorMessage = error.error.message;
    }
  }
}
