import {Injectable} from '@angular/core';
import {HttpClient, HttpParams} from '@angular/common/http';
import {Globals} from '../global/globals';
import {Observable} from 'rxjs';
import {Owner} from '../dto/owner';
import { Horse } from '../dto/horse';

@Injectable({
  providedIn: 'root'
})
export class OwnerService {

  private messageBaseUri: string = this.globals.backendUri + '/owners';

  constructor(private httpClient: HttpClient, private globals: Globals) {
  }

  /**
   * Loads specific owner from the backend
   * @param id of owner to load
   */
  getOwnerById(id: number): Observable<Owner> {
    console.log('Load owner details for ' + id);
    return this.httpClient.get<Owner>(this.messageBaseUri + '/' + id);
  }

  /**
   * Loads all owners from the backend
   */
  getAllOwners(): Observable<Array<Owner>> {
    console.log('Load all owners');
    return this.httpClient.get<Array<Owner>>(this.messageBaseUri);
  }

  /**
   * Load all owners from the backend that correspond to a filter
   * @param filter to use
   */
  getFilteredOwners(filter: HttpParams): Observable<Array<Owner>> {
    console.log('Load all filtered owners');
    return this.httpClient.get<Array<Owner>>(this.messageBaseUri, { params: filter });
  }

  /**
   * Load all horses, owned by the specified owner, from the backend
   * @param id
   */
  getOwnerHorses(id: number): Observable<Array<Horse>> {
    console.log('Load all horses for owner ' + id);
    return this.httpClient.get<Array<Horse>>(this.messageBaseUri + '/' + id + '/horses');
  }

  /**
   * Adds a specific owner to the backend and loads it if successful
   * @param owner
   */
  addOwner(owner: Owner): Observable<Owner> {
    console.log('Add new owner ' + JSON.stringify(owner));
    return this.httpClient.post<Owner>(this.messageBaseUri, owner);
  }

  /**
   * Update a specific owner with the supplied values
   * @param owner
   */
  updateOwner(owner: Owner): Observable<Owner> {
    console.log('Update owner with id ' + owner.id + ': '+ JSON.stringify(owner));
    return this.httpClient.put<Owner>(this.messageBaseUri + '/' + owner.id, owner);
  }

  /**
   * Delete an owner from the backend
   * @param id
   */
  deleteOwner(id: number): Observable<Object> {
    console.log('Delete owner with id ' + id);
    return this.httpClient.delete(this.messageBaseUri + '/' + id);
  }
}
